﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;

namespace csharp_test
{
    /**
        thanks to:

        Relative path to unmanaged DLL [duplicate]
        http://stackoverflow.com/questions/9256901/relative-path-to-unmanaged-dll

        kernel32.GetDllDirectory returns error (0) when called via pinvoke
        http://stackoverflow.com/questions/19822156/kernel32-getdlldirectory-returns-error-0-when-called-via-pinvoke
     */

    public class UnsafeNativeMethods
    {
        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern bool SetDllDirectory(string lpPathName);

        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern int GetDllDirectory(int bufsize, StringBuilder buf);

        [DllImport("kernel32", CharSet = CharSet.Unicode, SetLastError = true)]
        public static extern IntPtr LoadLibrary(string librayName);

        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern int GetCurrentDirectory(int bufsize, StringBuilder buf);
    }

    class DllLoader
    {
        public static void Load(string dllfolder, string libname)
        {
            var currentpath = new StringBuilder(255);
            UnsafeNativeMethods.GetDllDirectory(currentpath.Length, currentpath);

            // use new path
            UnsafeNativeMethods.SetDllDirectory(dllfolder);

            UnsafeNativeMethods.LoadLibrary(libname);

            // restore old path
            UnsafeNativeMethods.SetDllDirectory(currentpath.ToString());
        }
    }

    class Program
    {
        [DllImport("autorg_kek_glue.dll", CallingConvention = CallingConvention.Cdecl)]
        private extern static int Autorg_Initialize(char[] autorg_lib_dir);

        [DllImport("autorg_kek_glue.dll", CallingConvention = CallingConvention.Cdecl)]
        private extern static int Autorg_Execute(char[] data_file, char[] result_file, int ATSAS, int robust);

        [DllImport("autorg_kek_glue.dll", CallingConvention = CallingConvention.Cdecl)]
        private extern static int Autorg_Finalize();

        static void Main(string[] args)
        {
            // Locate the glue dll directory
            string current_dir_ = System.IO.Directory.GetCurrentDirectory();
            System.Console.WriteLine("current_dir_={0} \n", current_dir_);

            string[] current_dir_split = current_dir_.Split('\\');
            string dll_dir = string.Join("\\", current_dir_split.Take(current_dir_split.Length - 3)) + "\\x64\\Release";
            System.Console.WriteLine("dll_dir={0} \n", dll_dir);

            // Load the glue dll
            DllLoader.Load(dll_dir, "autorg_kek_glue");

            int ret;

            ret = Autorg_Initialize( "../../../../lib".ToCharArray() );
            // ret = Autorg_Initialize( "F:/Guinier/autorg/autorg_kek-0_3_4-x64/lib".ToCharArray() );
            System.Console.WriteLine( "Autorg_Initialize: ret={0} \n", ret.ToString() );

            ret = Autorg_Execute( "F:/Guinier/Data/test_data4/GI/GI_0.7_ave_sub.dat".ToCharArray(), "result_kek.csv".ToCharArray(), 0, 0 );
            System.Console.WriteLine( "Autorg_Execute: ret={0} \n", ret.ToString());

            ret = Autorg_Execute( "F:/Guinier/Data/test_data4/GI/GI_0.7_ave_sub.dat".ToCharArray(), "result_atsas.csv".ToCharArray(), 1, 0 );
            System.Console.WriteLine( "Autorg_Execute: ret={0} \n", ret.ToString());

            ret = Autorg_Execute( "F:/Guinier/Data/20160513autorg_error/Mei_subtRNA_00144_sub.dat".ToCharArray(), "result_error.csv".ToCharArray(), 0, 0 );
            System.Console.WriteLine( "Autorg_Execute: ret={0} \n", ret.ToString());

            ret = Autorg_Execute( "F:/Guinier/Data/20160513autorg_error/Mei_subtRNA_00144_sub.dat".ToCharArray(), "result_error_robust.csv".ToCharArray(), 0, 1 );
            System.Console.WriteLine( "Autorg_Execute: ret={0} \n", ret.ToString());

            Autorg_Finalize();
        }
    }
}
